//Author: Santiago Jimeno ( mailto:sjimeno@ya.com )

using System;
using System.Text;
using System.IO;

    class FLACInfo
    {
        private Encoding Encod = new UTF8Encoding();
        private FileStream sAUDIOFile;
        private string[] aNChan = new string[] { "Mono", "Stereo" };
        //From next fiels you can do the public properties you need
        private string m_sFilename;
        private long m_nFileSize;
        private string sVersion;
        private int nFrecuency;
        private int nChannel;
        private string sChannel;
        private int nBitsPerSample;
        private int nBytesPersample;
        private int nBlockAlign;
        private float fLength;
        private string sLength;
        private long nWavSize;//If int-> Error computing nBitrate
        private int nBitrate;
        private int nFirstFramePos;
        private int nFrames;
        private long nAudioSize;
        private string sCompress;
        //--------------------------------------------------------------------
        
        public bool LoadFLACHeader(string FileName) 
        //FLAC use bigendian and VORBIS_COMMENT use littleendian
        {
            byte[] TmpByte16 = new byte[2];
            byte[] TmpByte32 = new byte[4];
            byte[] TmpByte64 = new byte[8];

            if (Path.GetExtension(FileName)!= ".flac")
            {
                return false;
            }
            sAUDIOFile = new FileStream(FileName, FileMode.Open, FileAccess.Read);
            m_sFilename = FileName;
            m_nFileSize = sAUDIOFile.Length;

            sAUDIOFile.Read(TmpByte32, 0, 4);
            if (Encod.GetString(TmpByte32, 0, 4) != "fLaC")
            {
                return false;
            }
            sVersion = "FLAC ";
            sAUDIOFile.Read(TmpByte32, 0, 4);
            Array.Reverse(TmpByte32);//convert to littleendian
            int BlockHeader = BitConverter.ToInt32(TmpByte32, 0);
            if ((((BlockHeader >> 0x1f) & 1) != 0) | (((BlockHeader >> 0x18) & 0x7f) != 0))
            //Error->There is not StreamInfo Block
            {
                return false;
            }
            int nBlockSize = BlockHeader & 0x7fffff;

            sAUDIOFile.Read(TmpByte16, 0, 2);
            Array.Reverse(TmpByte16);
            short minBlock = BitConverter.ToInt16(TmpByte16, 0); 
            sAUDIOFile.Read(TmpByte16, 0, 2);
            Array.Reverse(TmpByte16);
            short maxBlock = BitConverter.ToInt16(TmpByte16, 0);
            sAUDIOFile.Position += 6;// skip Frame(min-max)
            sAUDIOFile.Read(TmpByte64, 0, 8);
            Array.Reverse(TmpByte64);
            long BlockInfo = BitConverter.ToInt64(TmpByte64, 0);
            nFrecuency = (int)((BlockInfo >> 0x2c) & 0xfffffL);//SampleRate
            nChannel = (int) (((BlockInfo >> 0x29) & 7L) + 1);
            sChannel = aNChan[nChannel - 1];
            nBitsPerSample = (int) (((BlockInfo >> 0x24) & 0x1fL) + 1);
            nBytesPersample = nBitsPerSample / 8;
            nBlockAlign = nBytesPersample * nChannel;
            long nTotalSamples = BlockInfo & 0xfffffffffL;
            if (minBlock == maxBlock)
            {
                nFrames = (int)nTotalSamples / minBlock;
            }
            fLength = (float) (((double) nTotalSamples) / ((double) nFrecuency));
            sLength = FormatDuration((int) Math.Round((double) fLength));
            nWavSize = nTotalSamples * nBlockAlign; 
            nBitrate = (int)Math.Round((double)(((float)(nWavSize * 8)) / (fLength * 1000)));//'in milliseconds
            sAUDIOFile.Position += 16;//skip MD5 Info
        
            //Look for Version & 1st Frame
            do
            {
                sAUDIOFile.Read(TmpByte32, 0, 4);
                Array.Reverse(TmpByte32);
                BlockHeader = BitConverter.ToInt32(TmpByte32, 0); //BlockHeader is bigendian
                if (((BlockHeader >> 0x18) & 0x7f) == 4) // Then VORBIS Block
                {
                    // From here (VORBIS) is littleendian
                    // I only read Vendor Data(version number) 
                    // If you want to read Tags then you should do the same as with Vendor 
                    // VORBIS specifications in http://www.xiph.org/vorbis/doc/v-comment.html
                    sAUDIOFile.Read(TmpByte32, 0, 4); 
                    int nVendor = BitConverter.ToInt32(TmpByte32, 0);//Size next tag
                    byte[] aVendor = new byte[(nVendor - 1) + 1];
                    sAUDIOFile.Read(aVendor, 0, aVendor.Length);
                    string sVendor = Encod.GetString(aVendor, 0, aVendor.Length);//Tag UTF-8 text
                    sVersion = sVersion + sVendor.Substring(0x12, 5);
                    sAUDIOFile.Position += (BlockHeader & 0x7fffff) - (nVendor + 4);
                }
                else
                {
                    sAUDIOFile.Position += BlockHeader & 0x7fffff;
                }
            }
            while (((BlockHeader >> 0x1f) & 1) != 1);//Last metadata block
        
            nFirstFramePos = (int) sAUDIOFile.Position;
            nAudioSize = m_nFileSize - nFirstFramePos;
            //Compression Ratio
            string sTemp1 = (((float)nAudioSize) * 100 / ((float)nWavSize)).ToString();
            string sTemp2 = (((float)nWavSize) / ((float)nAudioSize)).ToString(); 
            sCompress = sTemp1.Substring(0, 2) + "% (" + sTemp2.Substring(0, 3) + "/1)";
            // It lacks to make CompressionLevel? Default=5
            sAUDIOFile.Close();
            return true;
        }
        //--------------------------------------------------------------------
        
        private string FormatDuration(int Duration)
        {
            if (Duration >= 0)
            {
                int lHours = Duration / 0xe10;
                int lMinutes = (Duration - (lHours * 0xe10)) / 60;
                int lSeconds = (Duration - (lHours * 0xe10)) - (lMinutes * 60);

                if (lHours > 0)
                {
                    return lHours.ToString() + ":" + PadTwoDigits(lMinutes.ToString()) + ":" + PadTwoDigits(lSeconds.ToString());
                }
                return lMinutes.ToString() + ":" + PadTwoDigits(lSeconds.ToString());
            }
            return "";
        }
        //--------------------------------------------------------------------

        private string PadTwoDigits(string Text)
        {
            if (Text.Length == 1)
            {
                return ("0" + Text);
            }
            return Text;
        }
        //--------------------------------------------------------------------
    }
